@@ -9,7 +9,7 @@
  * Refer to the COPYING file of the official project for license.
  *****************************************************************************/
 
-void _notAMethod()
+void _notAMethod(void)
 {
     NSLocalizedString(@"STORE_DESCRIPTION", nil);
     NSLocalizedString(@"STORE_DESCRIPTION_TV", nil);
