@@ -1,7 +1,7 @@
 /*****************************************************************************
  * VLC for iOS
  *****************************************************************************
- * Copyright (c) 2015, 2020 VideoLAN. All rights reserved.
+ * Copyright (c) 2015, 2020-2021 VideoLAN. All rights reserved.
  * $Id$
  *
  * Authors: Felix Paul Kühne <fkuehne # videolan.org>
@@ -27,6 +27,8 @@
 @property (strong, nonatomic) VLCOSOFetcher *osoFetcher;
 @property (strong, nonatomic) NSArray<VLCSubtitleItem *>* searchResults;
 @property (strong, nonatomic) UIActivityIndicatorView *activityIndicatorView;
+@property (strong, nonatomic) UILabel *nothingFoundLabel;
+@property (nonatomic) BOOL activityCancelled;
 
 @end
 
@@ -44,6 +46,10 @@
 
     self.navigationItem.rightBarButtonItem = doneButton;
     self.tableView.separatorStyle = UITableViewCellSeparatorStyleNone;
+    if (@available(iOS 11.0, *)) {
+        self.navigationController.navigationBar.prefersLargeTitles = NO;
+        self.navigationItem.largeTitleDisplayMode = NO;
+    }
 #endif
 
     self.osoFetcher = [[VLCOSOFetcher alloc] init];
@@ -66,8 +72,13 @@
         [defaults setObject:selectedLocale forKey:kVLCSettingLastUsedSubtitlesSearchLanguage];
     }
     self.osoFetcher.subtitleLanguageId = selectedLocale;
-
-    self.activityIndicatorView = [[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleWhiteLarge];
+    self.activityIndicatorView = [[UIActivityIndicatorView alloc] init];
+    if (@available(iOS 13.0, *)) {
+        self.activityIndicatorView.activityIndicatorViewStyle = UIActivityIndicatorViewStyleLarge;
+    } else {
+        self.activityIndicatorView.activityIndicatorViewStyle = UIActivityIndicatorViewStyleWhiteLarge;
+    }
+    self.activityIndicatorView.color = [UIColor lightGrayColor];
     [self.activityIndicatorView sizeToFit];
     [self.activityIndicatorView setTranslatesAutoresizingMaskIntoConstraints:NO];
     self.activityIndicatorView.hidesWhenStopped = YES;
@@ -90,6 +101,54 @@
                                                                     constant:0.0];
     [self.view addConstraint:xConstraint];
 
+    self.nothingFoundLabel = [[UILabel alloc] initWithFrame:CGRectZero];
+#if TARGET_OS_TV
+    self.nothingFoundLabel.font = [UIFont italicSystemFontOfSize:32];
+#else
+    self.nothingFoundLabel.font = [UIFont italicSystemFontOfSize:16];
+#endif
+    self.nothingFoundLabel.hidden = YES;
+    self.nothingFoundLabel.text = [NSString stringWithFormat:NSLocalizedString(@"NO_SUB_FOUND_OSO", nil), [VLCPlaybackService sharedInstance].metadata.title];
+    self.nothingFoundLabel.numberOfLines = 0;
+    self.nothingFoundLabel.textAlignment = NSTextAlignmentCenter;
+    [self.nothingFoundLabel sizeToFit];
+    [self.nothingFoundLabel setTranslatesAutoresizingMaskIntoConstraints:NO];
+    self.nothingFoundLabel.textColor = [UIColor blackColor];
+    [self.view addSubview:self.nothingFoundLabel];
+
+    yConstraint = [NSLayoutConstraint constraintWithItem:self.nothingFoundLabel
+                                                                   attribute:NSLayoutAttributeCenterY
+                                                                   relatedBy:NSLayoutRelationEqual
+                                                                      toItem:self.view
+                                                                   attribute:NSLayoutAttributeCenterY
+                                                                  multiplier:1.0
+                                                                    constant:0.0];
+    [self.view addConstraint:yConstraint];
+    xConstraint = [NSLayoutConstraint constraintWithItem:self.nothingFoundLabel
+                                                                   attribute:NSLayoutAttributeCenterX
+                                                                   relatedBy:NSLayoutRelationEqual
+                                                                      toItem:self.view
+                                                                   attribute:NSLayoutAttributeCenterX
+                                                                  multiplier:1.0
+                                                                    constant:0.0];
+    [self.view addConstraint:xConstraint];
+    NSLayoutConstraint *leftConstraint = [NSLayoutConstraint constraintWithItem:self.nothingFoundLabel
+                                                                   attribute:NSLayoutAttributeLeading
+                                                                   relatedBy:NSLayoutRelationGreaterThanOrEqual
+                                                                      toItem:self.view
+                                                                   attribute:NSLayoutAttributeLeading
+                                                                  multiplier:1.0
+                                                                    constant:20.0];
+    [self.view addConstraint:leftConstraint];
+    NSLayoutConstraint *rightConstraint = [NSLayoutConstraint constraintWithItem:self.nothingFoundLabel
+                                                                   attribute:NSLayoutAttributeTrailing
+                                                                   relatedBy:NSLayoutRelationGreaterThanOrEqual
+                                                                      toItem:self.view
+                                                                   attribute:NSLayoutAttributeTrailing
+                                                                  multiplier:1.0
+                                                                    constant:20.0];
+    [self.view addConstraint:rightConstraint];
+
 #if TARGET_OS_IOS
     [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(themeDidChange) name:kVLCThemeDidChangeNotification object:nil];
 #endif
@@ -113,8 +172,7 @@
         }
 
         if ([[NSUserDefaults standardUserDefaults] integerForKey:kVLCSettingAppTheme] == kVLCSettingAppThemeSystem) {
-            BOOL isSystemDarkTheme = self.traitCollection.userInterfaceStyle == UIUserInterfaceStyleDark;
-            PresentationTheme.current = isSystemDarkTheme ? PresentationTheme.darkTheme : PresentationTheme.brightTheme;
+            [PresentationTheme themeDidUpdate];
         }
         [self themeDidChange];
     }
@@ -133,14 +191,17 @@
     if ([UIScreen mainScreen].traitCollection.userInterfaceStyle == UIUserInterfaceStyleDark) {
         self.visualEffectView.effect = [UIBlurEffect effectWithStyle:UIBlurEffectStyleDark];
         self.titleLabel.textColor = [UIColor VLCLightTextColor];
+        self.nothingFoundLabel.textColor = [UIColor VLCLightTextColor];
     } else {
         self.visualEffectView.effect = [UIBlurEffect effectWithStyle:UIBlurEffectStyleLight];
         self.titleLabel.textColor = [UIColor VLCDarkTextColor];
+        self.nothingFoundLabel.textColor = [UIColor VLCDarkTextColor];
     }
 #else
     ColorPalette *colors = PresentationTheme.current.colors;
     [self.navigationItem.titleView setTintColor:colors.navigationbarTextColor];
-    self.view.backgroundColor = self.tableView.backgroundColor = colors.background;
+    self.nothingFoundLabel.backgroundColor = self.view.backgroundColor = self.tableView.backgroundColor = colors.background;
+    self.nothingFoundLabel.textColor = colors.cellTextColor;
     [self.tableView reloadData];
 #endif
 }
@@ -170,10 +231,33 @@
 
 - (void)VLCOSOFetcher:(VLCOSOFetcher *)aFetcher didFindSubtitles:(NSArray<VLCSubtitleItem *> *)subtitles forSearchRequest:(NSString *)searchRequest
 {
-    APLog(@"%s: %li items found", __func__, subtitles.count);
+    NSUInteger count = subtitles.count;
+    APLog(@"%s: %li items found", __func__, count);
     [self stopActivity];
     self.searchResults = subtitles;
     [self.tableView reloadData];
+
+    if (count == 0) {
+        self.nothingFoundLabel.text = [NSString stringWithFormat:NSLocalizedString(@"NO_SUB_FOUND_OSO", nil), [VLCPlaybackService sharedInstance].metadata.title];
+        self.nothingFoundLabel.hidden = NO;
+    } else {
+        self.nothingFoundLabel.hidden = YES;
+    }
+}
+
+- (void)VLCOSOFetcher:(VLCOSOFetcher * _Nonnull)aFetcher didFailToFindSubtitlesForSearchRequest:(NSString * _Nonnull)searchRequest
+{
+    if (!self.activityCancelled) {
+        return;
+    }
+    APLog(@"%s: failed to find subtitles for request %@", __func__, searchRequest);
+    [self stopActivity];
+    self.searchResults = @[];
+    [self.tableView reloadData];
+
+    self.nothingFoundLabel.text = [NSString stringWithFormat:NSLocalizedString(@"NO_SUB_FOUND_OSO", nil), [VLCPlaybackService sharedInstance].metadata.title];
+    self.nothingFoundLabel.hidden = NO;
+    self.activityCancelled = NO;
 }
 
 - (void)VLCOSOFetcher:(VLCOSOFetcher *)aFetcher didFailToDownloadForItem:(VLCSubtitleItem *)subtitleItem
@@ -238,10 +322,11 @@
     }
 
 #if TARGET_OS_IOS
-    cell.backgroundColor = (indexPath.row % 2 == 0)? PresentationTheme.current.colors.cellBackgroundA : PresentationTheme.current.colors.cellBackgroundB;
-    cell.selectedBackgroundView.backgroundColor = PresentationTheme.current.colors.mediaCategorySeparatorColor;
-    cell.textLabel.textColor = PresentationTheme.current.colors.cellTextColor;
-    cell.detailTextLabel.textColor = PresentationTheme.current.colors.cellDetailTextColor;
+    ColorPalette *colors = PresentationTheme.current.colors;
+    cell.backgroundColor = (indexPath.row % 2 == 0)? colors.cellBackgroundA : colors.cellBackgroundB;
+    cell.selectedBackgroundView.backgroundColor = colors.mediaCategorySeparatorColor;
+    cell.textLabel.textColor = colors.cellTextColor;
+    cell.detailTextLabel.textColor = colors.cellDetailTextColor;
 #endif
 
     return cell;
@@ -253,7 +338,9 @@
         return @"";
     }
 
-    return NSLocalizedString(@"FOUND_SUBS", nil);
+    return section == 1 && self.searchResults.count > 0
+    ? NSLocalizedString(@"FOUND_SUBS", nil)
+    : @"";
 }
 
 #pragma mark - table view delegate
@@ -324,11 +411,20 @@
 - (void)startActivity
 {
     [self.activityIndicatorView startAnimating];
+    self.activityCancelled = NO;
+    [self performSelector:@selector(cancelActivity) withObject:nil afterDelay:5.];
 }
 
 - (void)stopActivity
 {
+    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(cancelActivity) object:nil];
     [self.activityIndicatorView stopAnimating];
 }
 
+- (void)cancelActivity
+{
+    self.activityCancelled = YES;
+    [self VLCOSOFetcher:self.osoFetcher didFailToFindSubtitlesForSearchRequest:@"cancellation request"];
+}
+
 @end
